==> bot-fw-custom-actions-v29-1.0.0.tgz: package/src/botFrameworkAttachmentCustomActions.ts <==
import {
  Expression,
  NumberExpression,
  NumberExpressionConverter,
  StringExpression,
  StringExpressionConverter,
} from "adaptive-expressions";
import { Count } from "adaptive-expressions/lib/builtinFunctions";

import {
  Converter,
  ConverterFactory,
  Dialog,
  DialogConfiguration,
  DialogContext,
  DialogTurnResult,
} from "botbuilder-dialogs";
const axios = require('axios');

import { MicrosoftAppCredentials } from "botframework-connector";
// import { AttachmentActions } from "./actions/attachmentActions";

export interface BotFrameworkAttachmentCustomActionsConfiguration extends DialogConfiguration {
  url: string | string | Expression | StringExpression;
  contentType: string | string | Expression | StringExpression;
  action: string | string | Expression | StringExpression;
  channelId: string | string | Expression | StringExpression;
  msAppId: string | string | Expression | StringExpression;
  msAppPwd: string | string | Expression | StringExpression;
  resultProperty?: string | Expression | StringExpression;
}

export class BotFrameworkAttachmentCustomActions
  extends Dialog
  implements BotFrameworkAttachmentCustomActionsConfiguration {
  public static $kind = "BotbuilderSamples.BotFrameworkAttachmentCustomActions";

  public url: StringExpression = new StringExpression('');
  public contentType: StringExpression = new StringExpression('');
  public action: StringExpression = new StringExpression('');
  public channelId: StringExpression = new StringExpression('');
  public msAppId: StringExpression = new StringExpression('');
  public msAppPwd: StringExpression = new StringExpression('');
  public resultProperty?: StringExpression;

  public getConverter(
    property: keyof BotFrameworkAttachmentCustomActionsConfiguration
  ): Converter | ConverterFactory {
    switch (property) {
      case "url":
        return new StringExpressionConverter();
      case "contentType":
        return new StringExpressionConverter();
      case "action":
        return new StringExpressionConverter();
      case "channelId":
        return new StringExpressionConverter();
      case "msAppId":
        return new StringExpressionConverter();
      case "msAppPwd":
        return new StringExpressionConverter();
      case "resultProperty":
        return new StringExpressionConverter();
      default:
        return super.getConverter(property);
    }
  }

  public async beginDialog(dc: DialogContext): Promise<DialogTurnResult> {
    const action = this.action.getValue(dc.state);
    let result;
    let debugStr = "|| 1 || ";
    try {
      if (action === 'GET_BASE64_CONTENT') {
        let token: string = '';
        let url = this.url.getValue(dc.state), urlStr: string = "";
        if (typeof url !== "string") {
          debugStr = debugStr + " || 1.1.1 ||";
          urlStr = JSON.stringify(url);
        } else {
          debugStr = debugStr + " || 1.1.2 ||";
          urlStr = url;
        }
        const typeStr = typeof urlStr, typeStr2 = typeof JSON.stringify(urlStr);
        debugStr = debugStr + " || 1.2 || typeStr: " + typeStr + " typeStr2: " + typeStr2;
        const channelId = this.channelId.getValue(dc.state);
        debugStr = debugStr + " || 2 || channelId: " + channelId;
        if (channelId == 'msteams' || channelId == 'skype') {
          debugStr = debugStr + " || 3 || app id: " + this.msAppId.getValue(dc.state);
          const credentials = new MicrosoftAppCredentials(this.msAppId.getValue(dc.state), this.msAppPwd.getValue(dc.state));
          debugStr = debugStr + " || 3.1 || b4 getToken!";
          token = await credentials.getToken(false);
        }
        debugStr = debugStr + " || 4 || urlStr: " + urlStr + " STRINGIFIED: " + JSON.stringify(urlStr);

        let response, headers = {}, headersWithAuth = { 'Authorization': `Bearer ${token}` };        
        if (token) {
          response = await axios.get(urlStr, {headers: headersWithAuth, responseType: 'arraybuffer'});//, { responseType: 'arraybuffer' }); blob  
          debugStr = debugStr + " || 5.1 || response: " +  JSON.stringify(response);
        } else {
          response = await axios.get(urlStr, {headers: headers, responseType: 'arraybuffer'});//, { responseType: 'arraybuffer' }); blob
          debugStr = debugStr + " || 5.2 || response: " +  JSON.stringify(response);
        }   
        // If user uploads JSON file, this prevents it from being written as "{"type":"Buffer","data":[123,13,10,32,32,34,108..."
        if (response.headers['content-type'] === 'application/json') {
          debugStr = debugStr + " || 6 ||";
            response.data = JSON.parse(response.data, (key, value) => {
                return value && value.type === 'Buffer' ? Buffer.from(value.data) : value;
            });
        }
        debugStr = debugStr + " || 7 ||";
        result = JSON.stringify({"status": "SUCCESS", "headersWithAuth": headersWithAuth, "headers": headers, "base64_content": Buffer.from(response.data).toString('base64')});
        //result = await AttachmentActions.getBase64Content(urlStr, this.contentType.getValue(dc.state), token);
      }

      debugStr = debugStr + " || 8 || b4 resultProperty";
      if (this.resultProperty) {
        dc.state.setValue(this.resultProperty.getValue(dc.state), result);
      }
    } catch (error) {
      let errorText = "Unknown"; // error under useUnknownInCatchVariables 
      if (typeof error === "string") {
        errorText = error.toUpperCase() // works, `e` narrowed to string
      } else if (error instanceof Error) {
        errorText = error.message // works, `e` narrowed to Error
      }
      if (this.resultProperty) {
        dc.state.setValue(this.resultProperty.getValue(dc.state), JSON.stringify({
          "status": "FAILED-1",
          "url" : this.url.getValue(dc.state),
          "error": errorText,
          "debugLog": debugStr
        }));
      }
    }

    return dc.endDialog(result);
  }

  protected onComputeId(): string {
    return "BotbuilderSamples.BotFrameworkAttachmentCustomActions";
  }
}

==> bot-fw-custom-actions-v30-1.0.0.tgz: package/src/botFrameworkAttachmentCustomActions.ts <==
import {
  Expression,
  NumberExpression,
  NumberExpressionConverter,
  StringExpression,
  StringExpressionConverter,
} from "adaptive-expressions";
import { Count } from "adaptive-expressions/lib/builtinFunctions";

import {
  Converter,
  ConverterFactory,
  Dialog,
  DialogConfiguration,
  DialogContext,
  DialogTurnResult,
} from "botbuilder-dialogs";
const axios = require('axios');

import { MicrosoftAppCredentials } from "botframework-connector";
// import { AttachmentActions } from "./actions/attachmentActions";

export interface BotFrameworkAttachmentCustomActionsConfiguration extends DialogConfiguration {
  url: string | string | Expression | StringExpression;
  contentType: string | string | Expression | StringExpression;
  action: string | string | Expression | StringExpression;
  channelId: string | string | Expression | StringExpression;
  msAppId: string | string | Expression | StringExpression;
  msAppPwd: string | string | Expression | StringExpression;
  resultProperty?: string | Expression | StringExpression;
}

export class BotFrameworkAttachmentCustomActions
  extends Dialog
  implements BotFrameworkAttachmentCustomActionsConfiguration {
  public static $kind = "BotbuilderSamples.BotFrameworkAttachmentCustomActions";

  public url: StringExpression = new StringExpression('');
  public contentType: StringExpression = new StringExpression('');
  public action: StringExpression = new StringExpression('');
  public channelId: StringExpression = new StringExpression('');
  public msAppId: StringExpression = new StringExpression('');
  public msAppPwd: StringExpression = new StringExpression('');
  public resultProperty?: StringExpression;

  public getConverter(
    property: keyof BotFrameworkAttachmentCustomActionsConfiguration
  ): Converter | ConverterFactory {
    switch (property) {
      case "url":
        return new StringExpressionConverter();
      case "contentType":
        return new StringExpressionConverter();
      case "action":
        return new StringExpressionConverter();
      case "channelId":
        return new StringExpressionConverter();
      case "msAppId":
        return new StringExpressionConverter();
      case "msAppPwd":
        return new StringExpressionConverter();
      case "resultProperty":
        return new StringExpressionConverter();
      default:
        return super.getConverter(property);
    }
  }

  public async beginDialog(dc: DialogContext): Promise<DialogTurnResult> {
    const action = this.action.getValue(dc.state);
    let result;
    let debugStr = "|| 1 || ";
    try {
      if (action === 'GET_BASE64_CONTENT') {
        let token: string = '';
        let url = this.url.getValue(dc.state), urlStr: string = "";
        if (typeof url !== "string") {
          debugStr = debugStr + " || 1.1.1 ||";
          urlStr = JSON.stringify(url);
        } else {
          debugStr = debugStr + " || 1.1.2 ||";
          urlStr = url;
        }
        const typeStr = typeof urlStr, typeStr2 = typeof JSON.stringify(urlStr);
        debugStr = debugStr + " || 1.2 || typeStr: " + typeStr + " typeStr2: " + typeStr2;
        const channelId = this.channelId.getValue(dc.state);
        debugStr = debugStr + " || 2 || channelId: " + channelId;
        if (channelId == 'msteams' || channelId == 'skype') {
          debugStr = debugStr + " || 3 || app id: " + this.msAppId.getValue(dc.state);
          const credentials = new MicrosoftAppCredentials(this.msAppId.getValue(dc.state), this.msAppPwd.getValue(dc.state));
          debugStr = debugStr + " || 3.1 || b4 getToken!";
          token = await credentials.getToken(false);
        }
        debugStr = debugStr + " || 4 || urlStr: " + urlStr;

        let response, headers = {}, headersWithAuth = { 'Authorization': `Bearer ${token}` };        
        if (token) {
          debugStr = debugStr + " || 5.1 || ";
          response = await axios.get(urlStr, {headers: headersWithAuth, responseType: 'arraybuffer'});//, { responseType: 'arraybuffer' }); blob  
          debugStr = debugStr + " || 5.2 || response: "; // +  JSON.stringify(response);
        } else {
          debugStr = debugStr + " || 5.3 || ";
          response = await axios.get(urlStr, {responseType: 'arraybuffer'});//, { responseType: 'arraybuffer' }); blob
          debugStr = debugStr + " || 5.4 || response: "; // +  JSON.stringify(response);
        }   
        // If user uploads JSON file, this prevents it from being written as "{"type":"Buffer","data":[123,13,10,32,32,34,108..."
        if (response.headers['content-type'] === 'application/json') {
          debugStr = debugStr + " || 6 ||";
            response.data = JSON.parse(response.data, (key, value) => {
                return value && value.type === 'Buffer' ? Buffer.from(value.data) : value;
            });
        }
        debugStr = debugStr + " || 7 ||";
        result = JSON.stringify({"status": "SUCCESS", "headersWithAuth": headersWithAuth, "headers": headers, "base64_content": Buffer.from(response.data).toString('base64')});
        //result = await AttachmentActions.getBase64Content(urlStr, this.contentType.getValue(dc.state), token);
      }

      debugStr = debugStr + " || 8 || b4 resultProperty";
      if (this.resultProperty) {
        dc.state.setValue(this.resultProperty.getValue(dc.state), result);
      }
    } catch (error) {
      let errorText = "Unknown"; // error under useUnknownInCatchVariables 
      if (typeof error === "string") {
        errorText = error.toUpperCase() // works, `e` narrowed to string
      } else if (error instanceof Error) {
        errorText = error.message // works, `e` narrowed to Error
      }
      if (this.resultProperty) {
        dc.state.setValue(this.resultProperty.getValue(dc.state), JSON.stringify({
          "status": "FAILED-1",
          "url" : this.url.getValue(dc.state),
          "error": errorText,
          "debugLog": debugStr
        }));
      }
    }

    return dc.endDialog(result);
  }

  protected onComputeId(): string {
    return "BotbuilderSamples.BotFrameworkAttachmentCustomActions";
  }
}
